/**
 * Utilities
 *
 * @class
 */
var Util = function() {};

/**
 * Retrieve value from object
 *
 * @param  {Object|Array} obj
 * @param  {String[]} 	  pathToProperty
 *
 * @return {*}
 */
Util.getNestedProperty = function(obj, pathToProperty) {
	var none = {};
	var got = pathToProperty.reduce(function(prev, next) {
		return prev[next] ? prev[next] : none;
	}, obj);
	return got === none ? undefined : got;
};

/**
 * Gets the value at path of object
 * If the resolved value is undefined, the defaultValue is returned in its place.
 *
 * @param  {Object|Array} obj            Object to field value
 * @param  {String[]} 	  pathToProperty Array containing path
 * @param  {*} 			  defaultValue   Value returned if path does not contain a value
 *
 * @return {*}                			 Value found on the object using the path
 */
Util.get = function(obj, pathToProperty, defaultValue) {
	var pathDoesNotExists = false;

	var value = obj;
	for (var i = 0; i < pathToProperty.length && !pathDoesNotExists; i++) {
		if (typeof value === 'object') {
			value = value[pathToProperty[i]];
		} else {
			pathDoesNotExists = true;
		}
	}

	return pathDoesNotExists || value === undefined ? defaultValue : value;
};

Util.sha1 = function(word) {
	return CryptoJS.SHA1(word).toString();
};

Util.inDevToolsMode = function() {
	return ~window.location.href.indexOf("devtools=true");
};

Util.groupBy = function(data, key, owner) {
	if (!owner.dataGroup) {
		owner.dataGroup = {};
		for (var i = data.length - 1; i >= 0; i--) {
			if (!owner.dataGroup[data[i][key]]) {
				owner.dataGroup[data[i][key]] = [];
			}
			owner.dataGroup[data[i][key]].push(data[i]);
		}
	}
	return owner.dataGroup;
};

Util.clone = function(obj, ignoreKeys) {
	var result = obj;
	if (Util.isObject(obj) && Util.isArray(ignoreKeys) && ignoreKeys.length) {
		var clone = {};
		for (var key in obj) {
			if (ignoreKeys.indexOf(key) === -1) {
				clone[key] = Util.isGenericObject(obj[key]) ? Util.clone(obj[key], ignoreKeys) : obj[key];
			}
		}
		return clone;
	}
	return angular.copy(obj);
};

Util.extend = function(dest, source, makeNewObject) {
	return Util.extendWithOverride(dest, source, [], makeNewObject);
};
Util.extendWithOverride = function(dest, source, overrodeProperties, makeNewObject) {
	if (!overrodeProperties) {
		overrodeProperties = [];
	}
	dest = !makeNewObject ? dest : Util.clone(dest);
	for (var key in source) {
		if (source.hasOwnProperty(key) && overrodeProperties.indexOf(key) === -1) {
			dest[key] = source[key];
		}
	}
	return dest;
};

Util.decorateFromConstructor = function decorateFromConstructor(thisArg, constructor, argumentsToConstructor) {
	constructor.apply(thisArg, argumentsToConstructor);
};

Util.evaluate = function($string) {
	var $a, $function_detector, $function_new, $ret, $string_ret, _i, _len;
	if (angular.isFunction($string)) {
		$string_ret = $string();
	} else {
		$string_ret = $string;
		$function_detector = /\@(.*?)\@/g;
		if ($string_ret) {
			$ret = $string_ret.toString().match($function_detector);
			if ($ret) {
				for (_i = 0, _len = $ret.length; _i < _len; _i++) {
					$a = $ret[_i];
					$function_new = $a.replace(/\@/g, '');
					/*jslint evil: true */
					eval('$return = ' + $function_new + ';');
					if ($string_ret.length === $a.length) {
						$string_ret = $return;
					} else {
						$string_ret = $string_ret.replace($a, $return);
					}
				}
			}
		}
	}
	return $string_ret;
};

Util.isArray = function(value) {
	return Array.isArray(value);
};

Util.isGenericObject = function(value) {
	return angular.isObject(value);
};

Util.isObject = function(value) {
	return Object.prototype.toString.call(value) === '[object Object]';
};

Util.isFunction = function(value) {
	return Object.prototype.toString.call(value) === '[object Function]';
};

Util.isUndefined = function(value) {
	return typeof(value) === "undefined";
};

Util.isNull = function(value) {
	return value === null;
};

Util.isUndefinedOrNull = function(value) {
	return Util.isUndefined(value) || Util.isNull(value);
};

Util.isBlank = function(value) {
	return value.toString().trim().length === 0;
};

Util.isEmptyOrBlank = function(value) {
	return Util.isUndefinedOrNull(value) || Util.isBlank(value);
};

Util.isEmptyObject = function(obj) {
	return Object.keys(obj).length === 0;
};

Util.objectHasAnyValue = function objectHasAnyValue(obj) {
	return !Util.isUndefinedOrNull(obj) && Object.keys(obj).some(function(key) {
		return !Util.isEmptyOrBlank(obj[key]);
	});
};

Util.stringStartsWith = function stringStartsWith(string, valueToSearch) {
	return string.substring(0, valueToSearch.length) === valueToSearch;
};

Util.stringEndsWith = function stringEndsWith(string, valueToSearch) {
	return valueToSearch === "" || string.substring(string.length - valueToSearch.length, string.length) === valueToSearch;
};

Util.escapeRegExpString = function escapeRegExpString(str) {
	return str.replace(/[|\\{}()[\]^$+*?.]/g, "\\$&");
};

Util.isFalse = function(value) {
	return value === false;
};

Util.isDate = function(value) {
	return toString.call(value) === '[object Date]';
};

Util.isString = function(value) {
	return typeof(value) === "string";
};

Util.isNumber = function(value) {
	return typeof(value) === "number";
};

Util.doSync = function(promiseProviders, lastCallback) {
	var currentPromise;
	var callLastCallback = function() {
		if (lastCallback) {
			lastCallback();
		}
	};
	promiseProviders.forEach(function(promise) {
		var nextPromise = function() {
			promise.processed = true;
			return promise();
		};
		if (!currentPromise) {
			promise.processed = true;
			currentPromise = promise();
		} else {
			currentPromise = currentPromise.then(nextPromise, nextPromise);
		}
	});
	if (currentPromise) {
		currentPromise = currentPromise.then(function() {
			if (lastCallback) {
				lastCallback();
			}
		}, function() {
			if (lastCallback) {
				lastCallback();
			}
		});
	} else {
		if (lastCallback) {
			lastCallback();
		}
	}
	return currentPromise;
};

Util.doAsync = function(callBack, time) {
	return setTimeout(function() {
		callBack();
	}, time);
};

Util.hasIndexdDBSuport = function() {
	if (Util.isUndefined(Util._hasIndexdDBSuport)) {
		window.indexedDB = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
		Util._hasIndexdDBSuport = !!window.indexedDB;
	}

	return Util._hasIndexdDBSuport;
};

Util.buildDebounceMethod = function(method, milisecondsToDelay, firstCall) {
	var timeout;
	firstCall = typeof firstCall != "boolean" ? false : firstCall;
	return function() {
		var obj = this,
			args = arguments;

		function delayed() {
			if (!firstCall) {
				method.apply(obj, args);
			}
			timeout = null;
		}
		if (timeout) {
			clearTimeout(timeout);
		} else if (firstCall) {
			method.apply(obj, args);
		}
		timeout = setTimeout(delayed, milisecondsToDelay || 100);
	};
};

Util.buildThrottleMethod = function buildThrottleMethod(method, milisecondsToDelay, scope) {
	var threshhold = milisecondsToDelay || 250;
	var lastTime,
		deferTimer;
	return function() {
		var context = scope || this;

		var currentTime = +new Date(),
			args = arguments;

		if (lastTime && currentTime < lastTime + threshhold) {
			clearTimeout(deferTimer);
			deferTimer = setTimeout(function() {
				lastTime = currentTime;
				method.apply(context, args);
			}, threshhold);
		} else {
			lastTime = currentTime;
			method.apply(context, args);
		}
	};
};

Util.sortBy = (function() {
	function extend(f) {
		f.sortBy = tb;
		return f;
	}

	function tb(y) {
		var x = this;
		return extend(function(a, b) {
			return x(a, b) || y(a, b);
		});
	}
	return extend;
})();

Util.initializeComponent = function(owner, validationKey, methods, initialize, eventEngine) {
	if (owner && !owner[validationKey]) {
		if (initialize) {
			initialize();
		}
		if (methods) {
			for (var eventName in methods) {
				if (!owner[eventName]) {
					eventEngine.createEvent(owner, eventName);
				}
				owner[eventName](methods[eventName]);
			}
		}
		owner[validationKey] = true;
	}
};

Util.getUrlParameter = function(name) {
	var searchString = window.location.search.substring(1);
	var variableArray = searchString.split('&');
	var results = [];
	var result = "";
	for (var i = 0; i < variableArray.length; i++) {
		var keyValuePair = variableArray[i].split('=');
		if (keyValuePair[0] === name) {
			results.push(decodeURIComponent(keyValuePair[1]).replace("+", " "));
		}
	}
	if (results.length === 1) {
		return results[0];
	} else if (results.length > 1) {
		result = results;
	}
	return result;
};

Util.parseToArray = function(data, toEmpty) {
	if (Util.isArray(data)) {
		return data;
	} else if (Util.isUndefinedOrNull(data)) {
		return [];
	} else {
		return toEmpty ? [] : [data];
	}
};

var _fixMask = function(value, mask) {
	if (!value || !mask) {
		return "";
	}
	// replace to view on grid
	mask = mask.replace('?', '');
	var buf = [],
		m = 0,
		maskLen = mask.length,
		v = 0,
		valLen = value.length,
		offset = 1,
		addMethod = 'push',
		resetPos = -1,
		lastMaskChar,
		check;
	lastMaskChar = maskLen - 1;
	check = function() {
		return m < maskLen && v < valLen;
	};
	var translationDef = Util.getFixMaskPatternsTranslation();
	while (check()) {
		var maskDigit = mask.charAt(m),
			valDigit = value.charAt(v),
			translation = translationDef[maskDigit];

		if (translation) {
			if (valDigit.match(translation.pattern)) {
				buf[addMethod](valDigit);
				if (translation.recursive) {
					if (resetPos === -1) {
						resetPos = m;
					} else if (m === lastMaskChar) {
						m = resetPos - offset;
					}

					if (lastMaskChar === resetPos) {
						m -= offset;
					}
				}
				m += offset;
			} else if (translation.optional) {
				m += offset;
				v -= offset;
			} else if (translation.fallback) {
				buf[addMethod](translation.fallback);
				m += offset;
				v -= offset;
			}
			v += offset;
		} else {
			buf[addMethod](maskDigit);
			if (valDigit === maskDigit) {
				v += offset;
			}

			m += offset;
		}
	}

	var lastMaskCharDigit = mask.charAt(lastMaskChar);
	if (maskLen === valLen + 1 && !translationDef[lastMaskCharDigit]) {
		buf.push(lastMaskCharDigit);
	}

	return buf.join('');
};

Util.getFixMaskPatternsTranslation = function getFixMaskPatternsTranslation() {
	return {
		'0': {
			pattern: /\d/
		},
		'9': {
			pattern: /\d/,
			optional: true
		},
		'#': {
			pattern: /\d/,
			recursive: true
		},
		'A': {
			pattern: /[a-zA-Z0-9]/
		},
		'*': {
			pattern: /[a-zA-Z0-9]/
		},
		'S': {
			pattern: /[a-zA-Z]/
		},
		'a': {
			pattern: /[a-zA-Z]/
		}
	};
};

Util.applyMask = function(value, mask, maskEngine, ZhUtil) {
	if (mask.type === "fix") {
		value = _fixMask(value, ZhUtil.evaluate(mask.params.mask));
	} else {
		var mappedMasks = maskEngine.getMappedMasks();
		var maskFormatMethod = mappedMasks[mask.type] ? mappedMasks[mask.type].format : null;
		if (maskFormatMethod) {
			value = maskFormatMethod.apply(maskEngine, [value, mask.params]);
		}
	}
	return value;
};

Util.clearValue = function clearValue(value, mask, maskEngine) {
	parsedValue = value;

	if (value && mask) {
		var maskMethod = mask.type;
		var mappedMasks = maskEngine.getMappedMasks();
		var maskParserMethod = mappedMasks[maskMethod].parser;

		if (maskParserMethod) {
			parsedValue = maskParserMethod.apply(maskEngine, [value, mask.params || {}]);
		}
	}

	return parsedValue;
};

var getFormats = function(defaultFormat, format) {
	var formats = [defaultFormat];
	if (format) {
		formats.unshift(format);
	}
	return formats;
};

Util.DATE_FORMAT = "L";
Util.TIME_FORMAT = "LTS";
Util.DATETIME_FORMAT = "L LTS";

Util.isDateGt = function(date1, date2, dateFormat) {
	var formats = getFormats(Util.DATE_FORMAT, dateFormat);
	return moment(date1, formats)._d > moment(date2, formats)._d;
};

Util.isDateGte = function(date1, date2, dateFormat) {
	var formats = getFormats(Util.DATE_FORMAT, dateFormat);
	return moment(date1, formats)._d >= moment(date2, formats)._d;
};

Util.isDateLt = function(date1, date2, dateFormat) {
	return !Util.isDateGte(date1, date2, dateFormat);
};

Util.isDateLte = function(date1, date2, dateFormat) {
	return !Util.isDateGt(date1, date2, dateFormat);
};

Util.isBetweenDate = function(start, end, date, dateFormat) {
	return Util.isDateGte(date, start, dateFormat) && Util.isDateLte(date, end, dateFormat);
};

Util.isDateEqual = function(date1, date2, dateFormat) {
	var formats = getFormats(Util.DATE_FORMAT, dateFormat);
	return moment(date1, formats).format('x') === moment(date2, formats).format('x');
};

Util.isHourGt = function(hour1, hour2, hourFormat) {
	var formats = getFormats(Util.TIME_FORMAT, hourFormat);
	return moment(hour1, formats).format("x") > moment(hour2, formats).format("x");
};

Util.isHourGte = function(hour1, hour2, hourFormat) {
	var formats = getFormats(Util.TIME_FORMAT, hourFormat);
	return moment(hour1, formats).format("x") >= moment(hour2, formats).format("x");
};

Util.isHourLt = function(hour1, hour2, dateFormat) {
	return !Util.isHourGte(hour1, hour2, dateFormat);
};

Util.isHourLte = function(hour1, hour2, dateFormat) {
	return !Util.isHourGt(hour1, hour2, dateFormat);
};

Util.isHourEqual = function(date1, date2, hourFormat) {
	var formats = getFormats(Util.TIME_FORMAT, hourFormat);
	return moment(date1, formats).format('x') === moment(date2, formats).format('x');
};

Util.isValidDate = function(date, dateFormat) {
	return moment(date, getFormats(Util.DATE_FORMAT, dateFormat)).isValid();
};

Util.isValidTime = function(time, hourFormat) {
	return moment(time, getFormats(Util.TIME_FORMAT, hourFormat)).isValid();
};

Util.isValidDateTime = function(datetime, datetimeFormat) {
	return moment(datetime, getFormats(Util.DATETIME_FORMAT, datetimeFormat)).isValid();
};

Util.roundDecimal = function(value, decimals) {
	var pow = Math.pow(10, decimals);
	return Math.round(value * pow) / pow;
};

Util.isPhone = function() {
	return Zepto.os.phone;
};
Util.isTablet = function() {
	return Zepto.os.tablet || navigator.platform == 'iPad';
};
Util.isMobile = function() {
	return Util.isPhone() || Util.isTablet();
};
Util.isDesktop = function() {
	return !Util.isMobile();
};

Util.selectElement = Util.buildDebounceMethod(function(elm) {
	elm.focus();
	try {
		elm[0].setSelectionRange(0, 9999);
	} catch (e) {
		elm.select();
	}
});

Util.getRightPath = function(componentName, prefix) {
	prefix = prefix ? prefix : "";
	var parts = componentName.split("#");
	var path = parts[0];
	var resourceName = parts[1];
	var url;
	if (resourceName) {
		url = path + "#" + prefix + resourceName;
	} else {
		url = prefix + componentName;
	}
	return url;
};

Util.getPackage = function(uri) {
	var pckg = "";
	if (uri) {
		uri = uri.split("#");
		pckg = uri.length > 1 ? uri[0] : "";
	}
	return pckg;
};

Util.getPackageUrl = function(uri) {
	var packageUrl = "";
	if (uri) {
		var uriParts = uri.split("#");
		packageUrl = uriParts.length > 1 ? uriParts[1] : "";
	}
	return packageUrl;
};

Util.mergeSort = function(array, comparisonCallback) {
	var length = array.length,
		mid = Math.floor(length * 0.5),
		left = array.slice(0, mid),
		right = array.slice(mid, length);
	if (length <= 1) {
		return array;
	}
	return merge(Util.mergeSort(left, comparisonCallback), Util.mergeSort(right, comparisonCallback), comparisonCallback);
};


function merge(left, right, comparisonCallback) {
	var result = [];
	while (left.length && right.length) {
		if (comparisonCallback(left[0], right[0]) <= 0) {
			result.push(left.shift());
		} else {
			result.push(right.shift());
		}
	}
	return result.concat(left).concat(right);
}

Util.toBoolean = function toBoolean(value) {
	if (typeof value === 'function') {
		value = true;
	} else if (value && value.length !== 0) {
		var v = angular.lowercase("" + value);
		value = !(v == 'f' || v == '0' || v == 'false' || v == 'no' || v == 'n' || v == '[]');
	} else {
		value = false;
	}
	return value;
};

Util.getContainerJsonName = function getJsonFileName(container) {
	return container.fileName.replace('.json', '');
};

Util.isArrayLike = function isArrayLike(obj) {
	if (obj === null || isWindow(obj)) {
		return false;
	}
	var length = obj.length;
	if (obj.nodeType === 1 && length) {
		return true;
	}
	return angular.isString(obj) || angular.isArray(obj) || length === 0 ||
		typeof length === 'number' && length > 0 && (length - 1) in obj;

	function isWindow(obj) {
		return obj && obj.document && obj.location && obj.alert && obj.setInterval;
	}
};

Util.range = function range(start, end) {
	if (Util.isUndefined(end)) {
		end = start;
		start = 0;
	}

	var rangeArray = [];
	for (var i = start; i < end; i++) {
		rangeArray.push(i);
	}

	return rangeArray;
};

Util.isClickOnEditableField = function(event) {
	return event && event.target &&
		($(event.target).hasClass('editable-content') || $(event.target).hasClass('inline-editable'));
};

Util.removeAccents = function(str){
	var accents    = 'ÀÁÂÃÄÅàáâãäåÒÓÔÕÕÖòóôõöÈÉÊËèéêëðÌÍÎÏìíîïÙÚÛÜùúûüÑñŠšŸÿýŽž';
	var accentsOut = "AAAAAAaaaaaaOOOOOOoooooEEEEeeeeeIIIIiiiiUUUUuuuuNnSsYyyZz";
	str = str.split('');
	var i, x;
	for (i = 0; i < str.length; i++) {
		if ((x = accents.indexOf(str[i])) != -1) {
			str[i] = accentsOut[x];
		}
	}
	return str.join('');
};

/**
 * Verify if a optinal value is defined. If not defined the
 * default value is applied.
 *
 * @function  validateOptionalValue
 * @private
 * @param	{*} value - Optinal value
 * @param	{*} defaultValue - Default value
 * @return	{*} retorned value
 */
Util.validateOptionalValue = function(value, defaultValue) {
	if(this.isUndefinedOrNull(value)) {
		return defaultValue;
	} else {
		return value;
	}
};

window.Util = Util;